==> cramr_ui_lib-0.0.4.tar.gz: lib/routes/verb_indicative_study_write_mode_form_state.dart <==
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_form_appbar.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_route.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class VerbIndicativeStudyWriteModeFormState
    extends State<VerbIndicativeStudyRoute> {
  ComparisonStatus _comparisonStatus = ComparisonStatus.not_started;

  List<String> actual = ['', '', '', '', '', ''];

  int _currentPerson = 0;

  late Icon statusIcon = _updatedIcon();

  bool _isFirstRound = true;

  List<TextEditingController> _actualText = [
    TextEditingController(),
    TextEditingController(),
    TextEditingController(),
    TextEditingController(),
    TextEditingController(),
    TextEditingController()
  ];

  String _adaptedPronoun(String isoLang, String pronoun, String expected) {
    //TODO refactor with Strategy or the likes
    if ("fr" == isoLang) {
      if ("je" == pronoun &&
          (expected.substring(0, 1) == 'a' ||
              expected.substring(0, 1) == 'e' ||
              expected.substring(0, 1) == 'i' ||
              expected.substring(0, 1) == 'o' ||
              expected.substring(0, 1) == 'u')) {
        return "j\'";
      }
    }
    return pronoun;
  }

  Widget _pronounCell(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return Padding(
        padding: const EdgeInsets.all(5.0),
        child: SizedBox(
          key: UniqueKey(),
          height: 40,
          width: 85,
          child: Text(
            "${_adaptedPronoun(testModel.targetLanguageIso, testModel.pronouns[person], testModel.verb.indicative[person])}",
            key: Key(testModel.pronouns[person]),
            textAlign: TextAlign.center,
            style: TextStyle(
              color: Colors.blue,
              fontSize: 25,
            ),
          ),
        ),
      );
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        title: "Verbomatique",
        theme: CustomTheme.lightTheme,
        home: SafeArea(
          key: UniqueKey(),
          child: Scaffold(
            key: UniqueKey(),
            appBar: VerbIndicativeStudyFormAppbar().asWidget(context),
            body: Column(
              children: [
                Card(
                  child: Column(
                    children: [
                      Card(
                        child: Row(
                          children: [
                            _pronounCell(context, _currentPerson),
                            _expected(context, _currentPerson),
                          ],
                        ),
                        elevation: 20,
                      ),
                      Card(
                        child: Row(
                          children: [
                            _pronounCell(context, _currentPerson),
                            _actualTextField(context, _currentPerson),
                            statusIcon,
                          ],
                        ),
                        elevation: 20,
                      ),
                      Visibility(
                        visible: (_comparisonStatus ==
                            ComparisonStatus.completely_correct),
                        child: IconButton(
                            key: Key('nextPersonKey'),
                            onPressed: () {
                              setState(() {
                                _actualText[_currentPerson] =
                                    TextEditingController();
                                if (_currentPerson == 4) {
                                  _isFirstRound = false;
                                }
                                _currentPerson = ++_currentPerson % 6;
                                _comparisonStatus =
                                    ComparisonStatus.not_started;
                                statusIcon = _updatedIcon();
                              });
                            },
                            icon: Icon(Icons.arrow_forward)),
                      ),
                    ],
                  ),
                  elevation: 15,
                ),
              ],
            ),
            floatingActionButton: Visibility(
              visible: !_isFirstRound,
              child: FloatingActionButton(
                key: Key('write_forward_button'),
                child: Icon(Icons.arrow_forward),
                onPressed: () {
                  Navigator.pushNamed(
                      context, '/verbIndicativeWriteModeTestRoute');
                },
              ),
            ),
          ),
        ));
  }

  Icon _updatedIcon() {
    return Icon(
      Icons.assignment_turned_in_outlined,
      key: Key('comparisonStatusIcon'),
      color: _updateIcon(),
    );
  }

  void _validateInput(BuildContext context, int person, String expected) {
    if (actual[person].length == 0) {
      _comparisonStatus = ComparisonStatus.not_started;
    } else if (actual[person].length > expected.length) {
      _comparisonStatus = ComparisonStatus.more_than_one_wrong;
    } else if (actual[person].length == expected.length) {
      if (actual[person] == expected) {
        _comparisonStatus = ComparisonStatus.completely_correct;
      }
    } else if (actual[person].length < expected.length) {
      if (expected.substring(0, actual[person].length) == actual[person]) {
        _comparisonStatus = ComparisonStatus.still_correct;
      } else {
        _comparisonStatus = ComparisonStatus.first_wrong;
      }
    }
  }

  Color _updateIcon() {
    switch (_comparisonStatus) {
      case ComparisonStatus.not_started:
        return Colors.grey;
      case ComparisonStatus.still_correct:
        return Colors.orange;
      case ComparisonStatus.completely_correct:
        return Colors.green;
      case ComparisonStatus.first_wrong:
        return Colors.red;
      case ComparisonStatus.more_than_one_wrong:
        return Colors.red;
    }
  }

  Widget _expected(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return Padding(
        padding: const EdgeInsets.all(5.0),
        child: SizedBox(
            height: 40,
            width: 170,
            child: Text(testModel.verb.indicative[person],
                key: Key('expectedVerb'),
                style: TextStyle(
                  color: Colors.green,
                  fontSize: 25,
                ))),
      );
    });
  }

  Widget _actualTextField(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return SizedBox(
        height: 40,
        width: 170,
        child: TextField(
          key: Key('actualVerb'),
          obscureText: false,
          autofocus: true,
          decoration: InputDecoration(
            border: OutlineInputBorder(),
          ),
          readOnly: false,
          controller: _actualText[_currentPerson],
          onChanged: (input) {
            actual[_currentPerson] = _actualText[_currentPerson].text;
            _validateInput(
                context, _currentPerson, testModel.verb.indicative[person]);
            statusIcon = _updatedIcon();
            setState(() {});
          },
        ),
      );
    });
  }

  @override
  void dispose() {
    super.dispose();
    _actualText.forEach((element) {
      element.dispose();
    });
  }
}

enum ComparisonStatus {
  not_started,
  first_wrong,
  more_than_one_wrong,
  still_correct,
  completely_correct
}

==> cramr_ui_lib-0.0.5.tar.gz: lib/routes/verb_indicative_study_write_mode_form_state.dart <==
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_form_appbar.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_route.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class VerbIndicativeStudyWriteModeFormState
    extends State<VerbIndicativeStudyRoute> {
  ComparisonStatus _comparisonStatus = ComparisonStatus.not_started;

  List<String> actual = ['', '', '', '', '', ''];

  int _currentPerson = 0;

  late Icon statusIcon = _updatedIcon();

  bool _isFirstRound = true;

  List<TextEditingController> _actualText = [
    TextEditingController(),
    TextEditingController(),
    TextEditingController(),
    TextEditingController(),
    TextEditingController(),
    TextEditingController()
  ];

  String _adaptedPronoun(String isoLang, String pronoun, String expected) {
    //TODO refactor with Strategy or the likes
    if ("fr" == isoLang) {
      if ("je" == pronoun &&
          (expected.substring(0, 1) == 'a' ||
              expected.substring(0, 1) == 'e' ||
              expected.substring(0, 1) == 'i' ||
              expected.substring(0, 1) == 'o' ||
              expected.substring(0, 1) == 'u')) {
        return "j\'";
      }
    }
    return pronoun;
  }

  Widget _pronounCell(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return Padding(
        padding: const EdgeInsets.all(5.0),
        child: SizedBox(
          key: UniqueKey(),
          height: 40,
          width: 85,
          child: Text(
            "${_adaptedPronoun(testModel.targetLanguageIso, testModel.pronouns[person], testModel.verb.indicative[person])}",
            key: Key(testModel.pronouns[person]),
            textAlign: TextAlign.center,
            style: TextStyle(
              color: Colors.blue,
              fontSize: 25,
            ),
          ),
        ),
      );
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
        title: "Verbomatique",
        theme: CustomTheme.lightTheme,
        home: SafeArea(
          key: UniqueKey(),
          child: Scaffold(
            key: UniqueKey(),
            appBar: VerbIndicativeStudyFormAppbar().asWidget(context),
            body: Column(
              children: [
                Card(
                  child: Column(
                    children: [
                      Card(
                        child: Row(
                          children: [
                            _pronounCell(context, _currentPerson),
                            _expected(context, _currentPerson),
                          ],
                        ),
                        elevation: 20,
                      ),
                      Card(
                        child: Row(
                          children: [
                            _pronounCell(context, _currentPerson),
                            _actualTextField(context, _currentPerson),
                            _tapableIcon(context, _currentPerson),
                          ],
                        ),
                        elevation: 20,
                      ),
                      Visibility(
                        visible: (_comparisonStatus ==
                            ComparisonStatus.completely_correct),
                        child: IconButton(
                            key: Key('nextPersonKey'),
                            onPressed: () {
                              setState(() {
                                _actualText[_currentPerson] =
                                    TextEditingController();
                                if (_currentPerson == 4) {
                                  _isFirstRound = false;
                                }
                                _currentPerson = ++_currentPerson % 6;
                                _comparisonStatus =
                                    ComparisonStatus.not_started;
                                statusIcon = _updatedIcon();
                              });
                            },
                            icon: Icon(Icons.arrow_forward)),
                      ),
                    ],
                  ),
                  elevation: 15,
                ),
              ],
            ),
            floatingActionButton: Visibility(
              visible: !_isFirstRound,
              child: FloatingActionButton(
                key: Key('write_forward_button'),
                child: Icon(Icons.arrow_forward),
                onPressed: () {
                  Navigator.pushNamed(
                      context, '/verbIndicativeWriteModeTestRoute');
                },
              ),
            ),
          ),
        ));
  }

  Icon _updatedIcon() {
    return Icon(
      Icons.assignment_turned_in_outlined,
      key: Key('comparisonStatusIcon'),
      color: _updateIconColor(),
    );
  }

  void _validateInput(BuildContext context, int person, String expected) {
    if (actual[person].length == 0) {
      _comparisonStatus = ComparisonStatus.not_started;
    } else if (actual[person].length > expected.length) {
      _comparisonStatus = ComparisonStatus.more_than_one_wrong;
    } else if (actual[person].length == expected.length) {
      if (actual[person] == expected) {
        _comparisonStatus = ComparisonStatus.completely_correct;
      }
    } else if (actual[person].length < expected.length) {
      if (expected.substring(0, actual[person].length) == actual[person]) {
        _comparisonStatus = ComparisonStatus.still_correct;
      } else {
        _comparisonStatus = ComparisonStatus.first_wrong;
      }
    }
  }

  Color _updateIconColor() {
    switch (_comparisonStatus) {
      case ComparisonStatus.not_started:
        return Colors.grey;
      case ComparisonStatus.still_correct:
        return Colors.orange;
      case ComparisonStatus.completely_correct:
        return Colors.green;
      case ComparisonStatus.first_wrong:
        return Colors.red;
      case ComparisonStatus.more_than_one_wrong:
        return Colors.red;
    }
  }

  Widget _tapableIcon(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return InkWell(
        key: Key('tappableStatus'),
        child: statusIcon,
        onTap: () {
          setState(() {
            _validateInput(
                context, _currentPerson, testModel.verb.indicative[person]);
            statusIcon = _updatedIcon();
          });
        },
      );
    });
  }

  Widget _expected(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return Padding(
        padding: const EdgeInsets.all(5.0),
        child: SizedBox(
            height: 40,
            width: 170,
            child: Text(testModel.verb.indicative[person],
                key: Key('expectedVerb'),
                style: TextStyle(
                  color: Colors.green,
                  fontSize: 25,
                ))),
      );
    });
  }

  Widget _actualTextField(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return SizedBox(
        height: 40,
        width: 170,
        child: TextField(
          key: Key('actualVerb'),
          obscureText: false,
          autofocus: true,
          decoration: InputDecoration(
            border: OutlineInputBorder(),
          ),
          readOnly: false,
          controller: _actualText[_currentPerson],
          onChanged: (input) {
            actual[_currentPerson] = _actualText[_currentPerson].text;
            _validateInput(
                context, _currentPerson, testModel.verb.indicative[person]);
            statusIcon = _updatedIcon();
            if (input.length == testModel.verb.indicative[person].length) {
              setState(() {});
            }
          },
        ),
      );
    });
  }

  @override
  void dispose() {
    super.dispose();
    _actualText.forEach((element) {
      element.dispose();
    });
  }
}

enum ComparisonStatus {
  not_started,
  first_wrong,
  more_than_one_wrong,
  still_correct,
  completely_correct
}
